<?php
/**
* @package phpBB-WAP
* @copyright (c) phpBB Group
* @Оптимизация под WAP: Гутник Игорь ( чел ).
* @简体中文：中文phpBB-WAP团队
* @license http://opensource.org/licenses/gpl-license.php
**/

/**
* 这是一款自由软件, 您可以在 Free Software Foundation 发布的
* GNU General Public License 的条款下重新发布或修改; 您可以
* 选择目前 version 2 这个版本（亦可以选择任何更新的版本，由
* 你喜欢）作为新的牌照.
**/

/**
* 模版类
* 用于处理tpl文件
**/
class Template
{
	var $classname = "Template";
	var $_tpldata = array();
	var $files = array();
	var $root = "";
	var $compiled_code = array();
	var $uncompiled_code = array();
	var $fixed_style = '';
	
	/**
	* 初始化根目录
	**/
	function Template($root = '.', $fixed_style)
	{
		$this->set_rootdir($root);
		$this->fixed_style = $fixed_style;
	}

	/**
	* 初始化 $_tpldata
	**/
	function destroy()
	{
		$this->_tpldata = array();
	}

	/**
	* 设置成员属性root的值
	**/
	function set_rootdir($dir)
	{
		if (!is_dir($dir))
		{
			return false;
		}

		$this->root = $dir;
		return true;
	}

	/**
	* 选择 tpl 文件
	**/
	function set_filenames($filename_array)
	{
		// 判断是否同时加载多个模版
		if (!is_array($filename_array))
		{
			return false;
		}

		// 把指针指向第一个模版
		reset($filename_array);
		
		foreach($filename_array as $handle => $filename)
		{
			$this->files[$handle] = $this->make_filename($filename);
		}

		return true;
	}

	/**
	* 解析模版内容
	**/
	function pparse($handle)
	{
		// 首先加载代表的模版，无法加载则进行报错
		if (!$this->loadfile($handle))
		{
			trigger_error("Template->pparse(): Couldn't load template file for handle $handle", E_USER_ERROR);
		}

		// 检测模版是否编译，如果没有编译则进行编译
		if (!isset($this->compiled_code[$handle]) || empty($this->compiled_code[$handle]))
		{
			$this->compiled_code[$handle] = $this->compile($this->uncompiled_code[$handle]);
		}
		
		// 最后使用 eval 进行解析
		eval($this->compiled_code[$handle]);
		
		return true;
	}

	/*
	* 将模版解析在模版指定的标签中
	*/
	function assign_var_from_handle($varname, $handle)
	{
		if (!$this->loadfile($handle))
		{
			trigger_error("Template->assign_var_from_handle(): Couldn't load template file for handle $handle", E_USER_ERROR);
		}
		$_str = '';
		$code = $this->compile($this->uncompiled_code[$handle], true, '_str');
		eval($code);
		$this->assign_var($varname, $_str);

		return true;
	}

	/*
	* 指定区块内的文本值
	* @参数 字符串 $blockname 区块名称
	* @参数 数组 $vararray 标签和值
	*/
	function assign_block_vars($blockname, $vararray)
	{
		if (strstr($blockname, '.'))
		{
			$blocks = explode('.', $blockname);
			$blockcount = count($blocks) - 1;
			$str = '$this->_tpldata';
			for ($i = 0; $i < $blockcount; $i++)
			{
				$str .= '[\'' . $blocks[$i] . '.\']';
				eval('$lastiteration = count(' . $str . ') - 1;');
				$str .= '[' . $lastiteration . ']';
			}

			$str .= '[\'' . $blocks[$blockcount] . '.\'][] = $vararray;';

			eval($str);
		}
		else
		{

			$this->_tpldata[$blockname . '.'][] = $vararray;
		}

		return true;
	}

	/*
	* 指定若干个标签中的值 
	* @参数 数组 $vararray 标签和值
	*/
	function assign_vars($vararray)
	{
		reset($vararray);
		foreach($vararray as $key => $val)
		{
			$this->_tpldata['.'][0][$key] = $val;
		}

		return true;
	}
	
	/*
	* 指定单个标签中的值 
	* @参数 字符串 $varname 标签
	* @参数 字符串 $varval 标签的值
	*/
	function assign_var($varname, $varval)
	{
		$this->_tpldata['.'][0][$varname] = $varval;

		return true;
	}

	/**
	* 作用：生成标准的文件名
	* 如果模版文件不存在则使用 prosilver 下的模版
	* 文件，当 prosilver 都无此文件，则生成一个报
	* 错信息
	*
	* @参数 $filename 模版文件名
	**/
	function make_filename($filename)
	{
		if (substr($filename, 0, 1) != '/')
		{
       		$filename = ($rp_filename = phpbb_realpath($this->root . '/' . $filename)) ? $rp_filename : $filename;
		}

		if (!file_exists($filename))
		{
			$filename = $this->root . 'styles/' . $this->fixed_style . $filename;
			if( !file_exists($filename) )
			{
				trigger_error("Template->make_filename(): Error - file $filename does not exist", E_USER_ERROR);
			}
			return $filename;
		}
		
		return $filename;
	}

	/*
	* 生成未编译的代码
	*/
	function loadfile($handle)
	{
		if (isset($this->uncompiled_code[$handle]) && !empty($this->uncompiled_code[$handle]))
		{
			return true;
		}
		
		if (!isset($this->files[$handle]))
		{
			trigger_error("Template->loadfile(): No file specified for handle $handle", E_USER_ERROR);
		}

		$filename = $this->files[$handle];

		$str = implode('', @file($filename));
		
		if (empty($str))
		{
			trigger_error("Template->loadfile(): File $filename for handle $handle is empty", E_USER_ERROR);
		}

		$this->uncompiled_code[$handle] = $str;

		return true;
	}

	/*
	* 执行标签与真实值的替换 
	*/
	function compile($code, $do_not_echo = false, $retvar = '')
	{
		$code = str_replace('\\', '\\\\', $code);
		$code = str_replace('\'', '\\\'', $code);

		$varrefs = array();
		preg_match_all('#\{(([a-z0-9\-_]+?\.)+?)([a-z0-9\-_]+?)\}#is', $code, $varrefs);
		$varcount = count($varrefs[1]);
		for ($i = 0; $i < $varcount; $i++)
		{
			$namespace = $varrefs[1][$i];
			$varname = $varrefs[3][$i];
			$new = $this->generate_block_varref($namespace, $varname);

			$code = str_replace($varrefs[0][$i], $new, $code);
		}

		$code = preg_replace('#\{([a-z0-9\-_]*?)\}#is', '\' . ( ( isset($this->_tpldata[\'.\'][0][\'\1\']) ) ? $this->_tpldata[\'.\'][0][\'\1\'] : \'\' ) . \'', $code);
		$code_lines = explode("\n", $code);

		$block_nesting_level = 0;
		$block_names = array();
		$block_names[0] = ".";

		$line_count = count($code_lines);
		for ($i = 0; $i < $line_count; $i++)
		{
			$code_lines[$i] = chop($code_lines[$i]);
			if (preg_match('#<!-- BEGIN (.*?) -->#', $code_lines[$i], $m))
			{
				$n[0] = $m[0];
				$n[1] = $m[1];

				if ( preg_match('#<!-- END (.*?) -->#', $code_lines[$i], $n) )
				{
					$block_nesting_level++;
					$block_names[$block_nesting_level] = $m[1];
					if ($block_nesting_level < 2)
					{
						$code_lines[$i] = '$_' . $n[1] . '_count = ( isset($this->_tpldata[\'' . $n[1] . '.\']) ) ?  count($this->_tpldata[\'' . $n[1] . '.\']) : 0;';
						$code_lines[$i] .= "\n" . 'for ($_' . $n[1] . '_i = 0; $_' . $n[1] . '_i < $_' . $n[1] . '_count; $_' . $n[1] . '_i++)';
						$code_lines[$i] .= "\n" . '{';
					}
					else
					{
						$namespace = implode('.', $block_names);
						$namespace = substr($namespace, 2);
						$varref = $this->generate_block_data_ref($namespace, false);
						$code_lines[$i] = '$_' . $n[1] . '_count = ( isset(' . $varref . ') ) ? count(' . $varref . ') : 0;';
						$code_lines[$i] .= "\n" . 'for ($_' . $n[1] . '_i = 0; $_' . $n[1] . '_i < $_' . $n[1] . '_count; $_' . $n[1] . '_i++)';
						$code_lines[$i] .= "\n" . '{';
					}
					unset($block_names[$block_nesting_level]);
					$block_nesting_level--;
					$code_lines[$i] .= '} // END ' . $n[1];
					$m[0] = $n[0];
					$m[1] = $n[1];
				}
				else
				{
					$block_nesting_level++;
					$block_names[$block_nesting_level] = $m[1];
					if ($block_nesting_level < 2)
					{
						$code_lines[$i] = '$_' . $m[1] . '_count = ( isset($this->_tpldata[\'' . $m[1] . '.\']) ) ? count($this->_tpldata[\'' . $m[1] . '.\']) : 0;';
						$code_lines[$i] .= "\n" . 'for ($_' . $m[1] . '_i = 0; $_' . $m[1] . '_i < $_' . $m[1] . '_count; $_' . $m[1] . '_i++)';
						$code_lines[$i] .= "\n" . '{';
					}
					else
					{
						$namespace = implode('.', $block_names);
						$namespace = substr($namespace, 2);
						$varref = $this->generate_block_data_ref($namespace, false);
						$code_lines[$i] = '$_' . $m[1] . '_count = ( isset(' . $varref . ') ) ? count(' . $varref . ') : 0;';
						$code_lines[$i] .= "\n" . 'for ($_' . $m[1] . '_i = 0; $_' . $m[1] . '_i < $_' . $m[1] . '_count; $_' . $m[1] . '_i++)';
						$code_lines[$i] .= "\n" . '{';
					}
				}
			}
			else if (preg_match('#<!-- END (.*?) -->#', $code_lines[$i], $m))
			{
				unset($block_names[$block_nesting_level]);
				$block_nesting_level--;
				$code_lines[$i] = '} // END ' . $m[1];
			}
			else
			{
				if (!$do_not_echo)
				{
					$code_lines[$i] = 'echo \'' . $code_lines[$i] . '\' . "\\n";';
				}
				else
				{
					$code_lines[$i] = '$' . $retvar . '.= \'' . $code_lines[$i] . '\' . "\\n";'; 
				}
			}
		}

		$code = implode("\n", $code_lines);
		return $code;

	}

	/*
	* 生成块变量引用，编译时调用
	*/
	function generate_block_varref($namespace, $varname)
	{
		$namespace = substr($namespace, 0, strlen($namespace) - 1);
		$varref = $this->generate_block_data_ref($namespace, true);
		$varref .= '[\'' . $varname . '\']';
		$varref = '\' . ( ( isset(' . $varref . ') ) ? ' . $varref . ' : \'\' ) . \'';

		return $varref;

	}
	
	/*
	* 生成块数据引用，编译时调用
	*/
	function generate_block_data_ref($blockname, $include_last_iterator)
	{
		$blocks = explode(".", $blockname);
		$blockcount = count($blocks) - 1;
		$varref = '$this->_tpldata';
		for ($i = 0; $i < $blockcount; $i++)
		{
			$varref .= '[\'' . $blocks[$i] . '.\'][$_' . $blocks[$i] . '_i]';
		}
		$varref .= '[\'' . $blocks[$blockcount] . '.\']';
		if ($include_last_iterator)
		{
			$varref .= '[$_' . $blocks[$blockcount] . '_i]';
		}

		return $varref;
	}

}

?>