<?php
/**
* @package phpBB-WAP
* @copyright (c) phpBB Group
* @Оптимизация под WAP: Гутник Игорь ( чел ).
* @简体中文：中文phpBB-WAP团队
* @license http://opensource.org/licenses/gpl-license.php
**/

/**
* 这是一款自由软件, 您可以在 Free Software Foundation 发布的
* GNU General Public License 的条款下重新发布或修改; 您可以
* 选择目前 version 2 这个版本（亦可以选择任何更新的版本，由
* 你喜欢）作为新的牌照.
**/

 /**
 * 函数：get_db_stat(模式)
 * 说明：获取一些统计信息
 * 参数：
 *		（字符串）模式
 *			usercount 		统计用户 
 *			newestuser		统计新注册用户
 *			postcount		统计帖子
 *			topiccount		统计主题帖
 *			attachcount		统计附件
 * 返回：整型或布尔型
 **/
function get_db_stat($mode)
{
	global $db;
	switch( $mode )
	{
		case 'usercount':
			$sql = "SELECT COUNT(user_id) AS total
				FROM " . USERS_TABLE . "
				WHERE user_id <> " . ANONYMOUS;
			break;

		case 'newestuser':
			$sql = "SELECT user_id, username
				FROM " . USERS_TABLE . "
				WHERE user_id <> " . ANONYMOUS . "
				ORDER BY user_id DESC
				LIMIT 1";
			break;

		case 'postcount':
		case 'topiccount':
			$sql = "SELECT SUM(forum_topics) AS topic_total, SUM(forum_posts) AS post_total
				FROM " . FORUMS_TABLE;
			break;
		case 'attachcount':
			$sql = "SELECT count(*) AS total FROM " . ATTACHMENTS_DESC_TABLE;
			break;
	}
	if ( !($result = $db->sql_query($sql)) )
	{
		return false;
	}

	$row = $db->sql_fetchrow($result);

	switch ( $mode )
	{
		case 'usercount':
			return $row['total'];
			break;
		case 'newestuser':
			return $row;
			break;
		case 'postcount':
			return $row['post_total'];
			break;
		case 'topiccount':
			return $row['topic_total'];
			break;
		case 'attachcount':
			return $row['total'];
			break;
	}

	return false;
}

 /**
 * 函数：phpbb_clean_username(用户名)
 * 说明：规范用户名
 * 参数：
 *		（字符串）用户名
 * 返回：字符串 
 **/
function phpbb_clean_username($username)
{
	$username = substr(htmlspecialchars(str_replace("\'", "'", trim($username))), 0, 25);
	$username = phpbb_rtrim($username, "\\");
	$username = str_replace("'", "\'", $username);//添加反斜杠

	return $username;
}

 /**
 * 函数：phpbb_clean_username(邮件地址)
 * 说明：规范电子邮件
 * 参数：
 *		（字符串）
 * 返回：字符串
 **/
function phpbb_clean_email($email)
{
   $email = substr(htmlspecialchars(str_replace("\'", "'", trim($email))), 0, 255);
   $email = phpbb_rtrim($email, "\\");
   $email = str_replace("'", "\'", $email);

   return $email;
} 

/**
* phpbb_ltrim(被处理的字符串, 待删除的字符串)
* 说明：函数从字符串左侧删除空格或其他预定义字符。
* 参数：
*		待删除的字符串		当该值为空是默认为false
* 返回：字符串
**/
function phpbb_ltrim($str, $charlist = false)
{
	if ($charlist === false)
	{
		return ltrim($str);// ltrim() 函数从字符串左侧删除空格或其他预定义字符。
	}

	$str = ltrim($str, $charlist);

	return $str;
}

/**
* 函数：phpbb_ltrim(被处理的字符串, 待删除的字符串)
* 函数从字符串右端删除字符，用法同于phpbb_ltrim()函数，区别于左右
**/
function phpbb_rtrim($str, $charlist = false)
{
	if ($charlist === false)
	{
		return rtrim($str);
	}
	
	$str = rtrim($str, $charlist);
	
	return $str;
}

/**
* 函数：dss_rand()
* 作用：产生随机值
**/ 
function dss_rand()
{
	global $db, $board_config, $dss_seeded, $cache;

	$val = $board_config['rand_seed'] . microtime();
	$val = md5($val);
	$board_config['rand_seed'] = md5($board_config['rand_seed'] . $val . 'a');
   
	if($dss_seeded !== true)
	{
		$sql = "UPDATE " . CONFIG_TABLE . " SET
			config_value = '" . $board_config['rand_seed'] . "'
			WHERE config_name = 'rand_seed'";
		
		if( !$db->sql_query($sql) )
		{
			message_die(GENERAL_ERROR, "Unable to reseed PRNG", "", __LINE__, __FILE__, $sql);
		}
		$cache->remove_file('global_config');
		$dss_seeded = true;
	}

	return substr($val, 4, 16);
}

/**
* get_userdata(用户, 强制用户名模式)
* 说明：通过用户名或用户ID获取用户信息
* 参数：
*		强制模式 == ture	把“用户”参数的值强制定义为用户名
*		强制模式			默认为false  
**/
function get_userdata($user, $force_str = false)
{
	global $db;

	if (!is_numeric($user) || $force_str)
	{
		$user = phpbb_clean_username($user);
	}
	else
	{
		$user = intval($user);
	}

	$sql = "SELECT *
		FROM " . USERS_TABLE . " 
		WHERE ";
	$sql .= ( ( is_int($user) ) ? "user_id = $user" : "username = '" .  str_replace("\'", "''", $user) . "'" ) . " AND user_id <> " . ANONYMOUS;
	if ( !($result = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, '无法查询用户信息', '', __LINE__, __FILE__, $sql);
	}

	return ( $row = $db->sql_fetchrow($result) ) ? $row : false;
}

/**
* 初始化一些关于用户数据
**/
function init_userprefs($userdata)
{
	global $board_config, $template, $db;
	
	if ( $userdata['user_id'] != ANONYMOUS )
	{

		if ( !empty($userdata['user_dateformat']) )
		{
			$board_config['default_dateformat'] = $userdata['user_dateformat'];
		}
		// isset() 检测变量是否设置
		if ( isset($userdata['user_timezone']) )
		{
			$board_config['board_timezone'] = $userdata['user_timezone'];
		}
		if ( isset($userdata['user_topics_per_page']) )
		{
			$board_config['topics_per_page'] = $userdata['user_topics_per_page'];
		}
		if ( isset($userdata['user_posts_per_page']) )
		{
			$board_config['posts_per_page'] = $userdata['user_posts_per_page'];
		}
	}
	
	// 非匿名用户
	if ( $userdata['user_id'] != ANONYMOUS && $userdata['user_style'] > 0 )
	{
		setup_style($userdata['user_style']);
	}
	// 匿名用户
	else
	{
		setup_style($board_config['default_style']);
	}

	return;
}

/**
* 函数：setup_style(风格id)
* 说明：启动风格
* 参数：
*		(int)风格id		风格的id
*
* 参数类型：整数
**/
function setup_style($style)
{
	global $db, $board_config, $template, $global_style, $userdata, $global_style_path;
	
	$sql = 'SELECT style_path 
		FROM ' . STYLES_TABLE . ' 
		WHERE style_id = ' . $style;
		
	if (!$result = $db->sql_query($sql))
	{
		message_die(CRITICAL_ERROR, '风格加载出错', '', __LINE__, __FILE__, $sql);
	}
	
	if (!$row = $db->sql_fetchrow($result))
	{
		message_die(CRITICAL_MESSAGE, '数据库中无此风格的信息！', '', __LINE__, __FILE__, $sql);
	}

	$global_style = $style;
	$global_style_path = $row['style_path'];
	
	$template = new Template(ROOT_PATH . 'styles/' . $row['style_path'], 'prosilver');
}

/**
* 函数：encode_ip(ip地址)
* 说明：对ip进行十六进制编码
**/
function encode_ip($dotquad_ip, $die=true)
{
    //检测IPv6的localhost
	if ($dotquad_ip == "::1")
	{
		$dotquad_ip = '127.0.0.1';
	}
	
	if( !preg_match("/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})$/", $dotquad_ip) )
	{
		message_die(CRITICAL_ERROR, '这不是一个IP地址');
	}
	
	$ip_sep = explode('.', $dotquad_ip);
	
	return sprintf('%02x%02x%02x%02x', $ip_sep[0], $ip_sep[1], $ip_sep[2], $ip_sep[3]);
}

/**
* 函数：decode_ip(ip地址)
* 说明：把十六进制的ip转换成十进制
**/
function decode_ip($int_ip)
{
	$hexipbang = explode('.', chunk_split($int_ip, 2, '.'));
	return hexdec($hexipbang[0]). '.' . hexdec($hexipbang[1]) . '.' . hexdec($hexipbang[2]) . '.' . hexdec($hexipbang[3]);
}

/**
* 创建日期
**/
function create_date($format, $gmepoch, $tz)
{
	global $board_config;
	static $translate;

	if ( empty($translate) )
	{
		$datetime = array(
		'Sunday' => '星期日','Monday' => '星期一','Tuesday' => '星期二','Wednesday' => '星期三','Thursday' => '星期四','Friday' => '星期五','Saturday' => '星期六',

		'Sun' => '日','Mon' => '一', 'Tue' => '二','Wed' => '三', 'Thu' => '四','Fri' => '五', 'Sat' => '六',
		
		'January' => '1', 'February' => '2', 'March' => '3', 'April' => '4', 'Mays' => '5', 'June' => '6', 'July' => '7', 'August' => '8', 'September' => '9', 'October' => '10', 'November' => '11', 'December' => '12',

		'Jan' => '1', 'Feb' => '2', 'Mar' => '3', 'Apr' => '4', 'May' => '5', 'Jun' => '6', 'Jul' => '7', 'Aug' => '8', 'Sep' => '9', 'Oct' => '10', 'Nov' => '11', 'Dec' => '12',
		);
		
		@reset($datetime);
		foreach($datetime as $match => $replace)
		{
			$translate[$match] = $replace;
		}
	}
	return ( !empty($translate) ) ? strtr(@gmdate($format, $gmepoch + (3600 * $tz)), $translate) : @gmdate($format, $gmepoch + (3600 * $tz));
}

/**
* 创建分页处理
**/
function generate_pagination($base_url, $num_items, $per_page, $start_item, $add_prevnext_text = TRUE)
{
	// ceil() 函数向上舍入为最接近的整数
	$total_pages = ceil($num_items/$per_page);

	if ( $total_pages <= 1 )
	{
		return '';
	}
	// floor() 函数向下舍入为最接近的整数
	$on_page = floor($start_item / $per_page) + 1;

	$page_string = '<div class="pagination">';
	
	if ( $on_page == 1 )
	{
		// append_sid() 为phpBB-WAP内建函数，用于创建 SID 用
		$page_string = '<div>&lt;&lt;上一页 | <a href="' . append_sid($base_url . "&amp;start=" . ( $on_page * $per_page ) ) . '">下一页&gt;&gt;</a></div>';
	}
	if ( $on_page == $total_pages )
	{
		$page_string = '<div><a href="' . append_sid($base_url . "&amp;start=" . ( ( $on_page - 2 ) * $per_page ) ) . '">&lt;&lt;上一页</a> | 下一页&gt;&gt;<div>';
	}
	if ( $total_pages > 10 )
	{
		$init_page_max = ( $total_pages > 3 ) ? 3 : $total_pages;

		for($i = 1; $i < $init_page_max + 1; $i++)
		{
			$page_string .= ( $i == $on_page ) ? '<strong>' . $i . '</strong>' : '<a href="' . append_sid($base_url . "&amp;start=" . ( ( $i - 1 ) * $per_page ) ) . '">' . $i . '</a>';
			if ( $i <  $init_page_max )
			{
				$page_string .= ",";
			}
		}

		if ( $total_pages > 3 )
		{
			if ( $on_page > 1  && $on_page < $total_pages )
			{
				$page_string .= ( $on_page > 5 ) ? '...' : ',';

				$init_page_min = ( $on_page > 4 ) ? $on_page : 5;
				$init_page_max = ( $on_page < $total_pages - 4 ) ? $on_page : $total_pages - 4;

				for($i = $init_page_min - 1; $i < $init_page_max + 2; $i++)
				{
					$page_string .= ($i == $on_page) ? '<strong>' . $i . '</strong>' : '<a href="' . append_sid($base_url . "&amp;start=" . ( ( $i - 1 ) * $per_page ) ) . '">' . $i . '</a>';
					if ( $i <  $init_page_max + 1 )
					{
						$page_string .= ',';
					}
				}

				$page_string .= ( $on_page < $total_pages - 4 ) ? '...' : ',';
			}
			else
			{
				$page_string .= '...';
			}

			for($i = $total_pages - 2; $i < $total_pages + 1; $i++)
			{
				$page_string .= ( $i == $on_page ) ? '<strong>' . $i . '</strong>'  : '<a href="' . append_sid($base_url . "&amp;start=" . ( ( $i - 1 ) * $per_page ) ) . '">' . $i . '</a>';
				if( $i <  $total_pages )
				{
					$page_string .= ",";
				}
			}
		}
	}
	else
	{
		for($i = 1; $i < $total_pages + 1; $i++)
		{
			$page_string .= ( $i == $on_page ) ? '<strong>' . $i . '</strong>' : '<a href="' . append_sid($base_url . "&amp;start=" . ( ( $i - 1 ) * $per_page ) ) . '">' . $i . '</a>';
			if ( $i <  $total_pages )
			{
				$page_string .= ',';
			}
		}
	}

	if ( $add_prevnext_text )
	{
		if ( $on_page > 1  && $on_page < $total_pages )
		{
			$page_string = '<div><a href="' . append_sid($base_url . "&amp;start=" . ( ( $on_page - 2 ) * $per_page ) ) . '">&lt;&lt;上一页</a> | <a href="' . append_sid($base_url . "&amp;start=" . ( $on_page * $per_page ) ) . '">下一页&gt;&gt;</a></div>' . $page_string;
		}

		if ( $on_page < $total_pages )
		{
			$page_string .= '';
		}
	}

	if ( $total_pages > 3 )
	{	
		$select_list = '<div><form action="' . append_sid($base_url) . '" method="post">跳转到第 <input type="text" name="start1" size="3" value="' . $on_page . '"/> 页<input type="submit" value="&gt;&gt;"/></form></div>';
	}
	else
	{
		$select_list = '';
	}

	$page_number = '<div>第' . (floor( $start_item / intval($per_page) ) + 1)  . '页（共' . ceil( $num_items / intval($per_page) ) . '页）</div>';
	
	$page_string = $page_string . $select_list . $page_number . '</div>';

	return $page_string;
}

function phpbb_preg_quote($str, $delimiter)
{
	$text = preg_quote($str);
	$text = str_replace($delimiter, '\\' . $delimiter, $text);
	
	return $text;
}

/**
* 取得 word_list
**/
function obtain_word_list(&$orig_word, &$replacement_word)
{
	global $db;

	$sql = "SELECT word, replacement
		FROM  " . WORDS_TABLE;
	if( !($result = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, 'Could not get censored words from database', '', __LINE__, __FILE__, $sql);
	}

	if ( $row = $db->sql_fetchrow($result) )
	{
		do 
		{
			
			$orig_word[] = $row['word'];
			$replacement_word[] = $row['replacement'];
		}
		while ( $row = $db->sql_fetchrow($result) );
	}

	return true;
}

/**
* 函数：message_die(提示模式, 提示内容, 提示标题, 错误行数, 错误文件, 执行的SQL)
* 说明：输出一条消息，结束当前脚本！
* 参数：
*		
*
**/
function message_die($msg_code, $msg_text = '', $msg_title = '', $err_line = '', $err_file = '', $sql = '')
{
	global $db, $template, $global_style, $userdata, $user_ip, $board_config, $runtime, $modules;

	$sql_store = $sql;
	
	// 如果开启 DEBUG 模式和提示类型为错误类型则输出错误报告
	if ( DEBUG && ( $msg_code == GENERAL_ERROR || $msg_code == CRITICAL_ERROR ) )
	{
		// 得到错误类型
		$sql_error = $db->sql_error();

		$debug_text = '';

		// 如果 SQL 错误信息存在则输出提示
		if ( $sql_error['message'] != '' )
		{
			$debug_text .= '<p><strong>SQL 提示：</strong>' . $sql_error['code'] . ' ' . $sql_error['message'] . '</p>';
		}

		// 执行的 SQL 代码
		if ( $sql_store != '' )
		{
			$debug_text .= '<p><strong>执行的SQL：</strong>' . $sql_store . "</p>\n\t\t";
		}

		// 错误的行数和文件
		if ( $err_line != '' && $err_file != '' )
		{
			$debug_text .= '<p><strong>错误行数：</strong>' . $err_line . "</p>\n\t\t";
			$debug_text .= '<p><strong>错误文件：</strong>' . basename($err_file) . "</p>\n";
		}
	}

	if( empty($userdata) && ( $msg_code == GENERAL_MESSAGE || $msg_code == GENERAL_ERROR ) )
	{
		$userdata = session_pagestart($user_ip, PAGE_INDEX);
		init_userprefs($userdata);
	}

	switch($msg_code)
	{
		case GENERAL_MESSAGE:
			if ( $msg_title == '' )
			{
				$msg_title = '一般提示';
			}
			break;

		case CRITICAL_MESSAGE:
			if ( $msg_title == '' )
			{
				$msg_title = '重要提示';
			}
			break;

		case GENERAL_ERROR:
			if ( $msg_text == '' )
			{
				$msg_text = '发生错误';
			}

			if ( $msg_title == '' )
			{
				$msg_title = '发生错误';
			}
			break;

		case CRITICAL_ERROR:

			if ( $msg_text == '' )
			{
				$msg_text = '严重错误';
			}

			if ( $msg_title == '' )
			{
				$msg_title = '严重错误';
			}
			break;
	}

	if ( DEBUG && ( $msg_code == GENERAL_ERROR || $msg_code == CRITICAL_ERROR ) )
	{
		if ( $debug_text != '' )
		{
			$msg_text = '<p>' . $msg_text . "</p>\n\t\t<h2>调试模式</h2>\n\t\t" . $debug_text;
		}
	}

	if ( $msg_code != CRITICAL_ERROR )
	{
		$page_title = $msg_title;
		
		if ( !defined('IN_ADMIN') )
		{
			include(ROOT_PATH . 'includes/page_header.php');
		}
		else
		{
			include(ROOT_PATH . 'admin/page_header_admin.php');
		}

		if ( !defined('IN_ADMIN') )
		{
			$template->set_filenames(array(
				'message_body' => 'message_body.tpl')
			);
		}
		else
		{
			$template->set_filenames(array(
				'message_body' => 'admin/admin_message_body.tpl')
			);
		}

		$template->assign_vars(array(
			'MESSAGE_TITLE' 	=> $msg_title,
			'MESSAGE_TEXT' 		=> $msg_text)
		);
		$template->pparse('message_body');

		if ( !defined('IN_ADMIN') )
		{
			include(ROOT_PATH . 'includes/page_tail.php');
		}
		else
		{
			include(ROOT_PATH . 'admin/page_footer_admin.php');
		}
	}
	else
	{
		echo '<html>
	<head>
		<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
		<title>' . $msg_title . '</title>
		<style type="text/css">
			body {font-family: 微软雅黑, microsoft yahei;width: 320px;margin: 0 auto;margin-right: auto;line-height: 25px;}	
			h1, h2, p {margin: 0 auto;}
			h1 {font-size: 17px;text-align: center; color:red;}
			h2 {font-size: 14px;text-align: center; color:red;}
			p {font-size: 12px;color: #2d8d03;}
		</style>
	</head>
	<body>
		' . make_style_image('error') . '
		<h1>' . $msg_title . '</h1>
		' . $msg_text . '
	</body>
</html>';
	}
	exit;
}

/**
* 返回绝对路径
**/
function phpbb_realpath($path)
{
	// function_exists() 用来检查指定的函数是否已经定义
	return (!@function_exists('realpath') || !@realpath(ROOT_PATH . 'includes/functions/common.php')) ? $path : @realpath($path);
}

/**
* 测试一个文件和目录是否可写
*
* 当PHP本地函数 is_writable() 可以使用优先使用
*
* @参数 字符串 $file 文件路径
* @return 当可写时返回 true，否则返回 false.
*/
function phpbb_is_writable($file)
{
	if (strtolower(substr(PHP_OS, 0, 3)) === 'win' || !function_exists('is_writable'))
	{
		if (file_exists($file))
		{
			$file = phpbb_realpath($file);

			if (is_dir($file))
			{
				$result = @tempnam($file, 'i_w');

				if (is_string($result) && file_exists($result))
				{
					unlink($result);
					return (strpos($result, $file) === 0) ? true : false;
				}
			}
			else
			{
				$handle = @fopen($file, 'r+');

				if (is_resource($handle))
				{
					fclose($handle);
					return true;
				}
			}
		}
		else
		{
			$dir = dirname($file);

			if (file_exists($dir) && is_dir($dir) && phpbb_is_writable($dir))
			{
				return true;
			}
		}

		return false;
	}
	else
	{
		return is_writable($file);
	}
}

/**
* URL 重定向
* 旧的函数会因后台的域名、路径信息填写不正确而造成重定向失败
* 虽然这样做不是很理想，但是这是对一些新站长的一些照顾
**/
function redirect($url)
{
	global $db;

	//global $board_config;

	// 重定向之前先关闭数据库链接
	if (!empty($db))
	{
		$db->sql_close();
	}
	
	if (strstr(urldecode($url), "\n") || strstr(urldecode($url), "\r") || strstr(urldecode($url), ';url'))
	{
		message_die(GENERAL_ERROR, '由于该 URL 潜在安全隐患，因此您不能指向此 URL ');
	}

	/**
	//$server_protocol 	= ($board_config['cookie_secure']) ? 'https://' : 'http://';
	//$server_name 		= preg_replace('#^\/?(.*?)\/?$#', '\1', trim($board_config['server_name']));
	//$server_port 		= ($board_config['server_port'] <> 80) ? ':' . trim($board_config['server_port']) : '';
	//$script_name 		= preg_replace('#^\/?(.*?)\/?$#', '\1', trim($board_config['script_path']));
	//$script_name 		= ($script_name == '') ? $script_name : '/' . $script_name;
	//$url 				= preg_replace('#^\/?(.*?)\/?$#', '/\1', trim($url));
	**/
	
	header('Location: ' . ROOT_PATH . $url);
	exit;
}

/**
* 检查时间是否正确
**/
function mkrealdate($day, $month, $birth_year)
{
	if ($month < 1 || $month > 12)
	{
		return 'error';
	}
	
	switch ($month)
	{
		case 1:
			if ($day > 31)
			{
				return "error";
			}
			$epoch = 0;
			break;
		case 2:
			if ($day > 29)
			{
				return "error";
			}
			$epoch = 31;
			break;
		case 3:
			if ($day > 31)
			{
				return "error";
			}
			$epoch = 59;
			break;
		case 4:
			if ($day > 30)
			{
				return "error" ;
			}
			$epoch = 90;
			break;
		case 5:
			if ($day > 31)
			{
				return "error";
			}
			$epoch = 120;
			break;
		case 6:
			if ($day > 30)
			{
				return "error";
			}
			$epoch = 151;
			break;
		case 7:
			if ($day > 31)
			{
				return "error";
			}
			$epoch = 181;
			break;
		case 8:
			if ($day > 31)
			{
				return "error";
			}
			$epoch = 212;
			break;
		case 9:
			if ($day > 30)
			{
				return "error";
			}
			$epoch = 243;
			break;
		case 10:
			if ($day > 31)
			{
				return "error";
			}
			$epoch = 273;
			break;
		case 11:
			if ($day > 30)
			{
				return "error";
			}
			$epoch = 304;
			break;
		case 12:
			if ($day > 31)
			{
				return "error";
			}
			$epoch = 334;
			break;
	}
	$epoch 		= $epoch + $day;
	$epoch_Y 	= sqrt(($birth_year - 1970) * ($birth_year - 1970));// sqrt() 计算平方根
	$leapyear	=round((($epoch_Y+2) / 4)-.5);// round() 函数对浮点数进行四舍五入
	
	if (($epoch_Y + 2)%4 == 0)
	{
		$leapyear--;
		if ($birth_year > 1970 && $month>=3)
		{
			$epoch = $epoch + 1;
		}
		if ($birth_year < 1970 && $month < 3)
		{	
			$epoch=$epoch-1;
		}
	} 
	else if ($month==2 && $day>28)
	{	
		return "error";
	}
	if ($birth_year > 1970)
	{
		$epoch = $epoch + $epoch_Y * 365 - 1 + $leapyear;
	}
	else
	{
		$epoch =$epoch - $epoch_Y * 365 - 1 - $leapyear;
	}
	return $epoch;
}

/**
* 取得实际时间
**/
function realdate($date_syntax="Ymd",$date=0)
{
	$i = 2;
	if ($date >= 0)
	{
	 	return create_date($date_syntax, $date * 86400 + 1, 0);
	}
	else
	{
		$year= -(date%1461);
		$days = $date + $year*1461;
		while ($days<0)
		{
			$year--;
			$days+=365;
			if ($i++==3)
			{
				$i=0;
				$days++;
			}
		}
	}
	$leap_year = ($i==0) ? TRUE : FALSE;
	$months_array = ($i==0) ?
		array (0,31,60,91,121,152,182,213,244,274,305,335,366) :
		array (0,31,59,90,120,151,181,212,243,273,304,334,365);
	for ($month=1;$month<12;$month++)
	{
		if ($days<$months_array[$month])
		{
			break;//结束循环
		}
	}

	$day=$days-$months_array[$month-1]+1;
	$date_str = array(
		'day_short' => array(
			'datetime' => array(
				'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'
			)
		),
		'month_long' => array(
			'datetime' => array(
				'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'
			),
		),
		'day_long' => array(
			'datetime' => array(
				'Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'
			),
		),
		'month_short' => array(
			'datetime' => array(
				'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
			),
		)
	);
	return strtr ($date_syntax, array(
		'a' => '',
		'A' => '',
		'\\d' => 'd',
		'd' => ($day>9) ? $day : '0'.$day,
		'\\D' => 'D',
		'D' => $date_str['day_short'][($date-3)%7],
		'\\F' => 'F',
		'F' => $date_str['month_long'][$month-1],
		'g' => '',
		'G' => '',
		'H' => '',
		'h' => '',
		'i' => '',
		'I' => '',
		'\\j' => 'j',
		'j' => $day,
		'\\l' => 'l',
		'l' => $date_str['day_long'][($date-3)%7],
		'\\L' => 'L',
		'L' => $leap_year,
		'\\m' => 'm',
		'm' => ($month>9) ? $month : '0'.$month,
		'\\M' => 'M',
		'M' => $date_str['month_short'][$month-1],
		'\\n' => 'n',
		'n' => $month,
		'O' => '',
		's' => '',
		'S' => '',
		'\\t' => 't',
		't' => $months_array[$month]-$months_array[$month-1],
		'w' => '',
		'\\y' => 'y',
		'y' => ($year>29) ? $year-30 : $year+70,
		'\\Y' => 'Y',
		'Y' => $year+1970,
		'\\z' => 'z',
		'z' => $days,
		'\\W' => '',
		'W' => '') );
}

 /**
 * 函数：get_pagination_start(每页显示记录)
 * 说明：初始化分页的一些参数
 * 返回：整型
 **/
function get_pagination_start($per)
{
	if ( isset($_POST['start1']) )
	{
		$start1 = abs(intval($_POST['start1']));
		$start1 = ($start1 < 1) ? 1 : $start1;
		$start = (($start1 - 1) * $per);
	}
	else
	{
		$start = ( isset($_GET['start']) ) ? intval($_GET['start']) : 0;
		$start = ($start < 0) ? 0 : $start;
	}
	
	return $start;
}

/**
* 设置文件目录的权限
*
* This function determines owner and group whom the file belongs to and user and group of PHP and then set safest possible file permissions.
* The function determines owner and group from common.php file and sets the same to the provided file.
* The function uses bit fields to build the permissions.
* The function sets the appropiate execute bit on directories.
*
* 支持常量代表位字段:
*
* CHMOD_ALL - 所有权限 (7)
* CHMOD_READ - 读取权限 (4)
* CHMOD_WRITE - 写入权限 (2)
* CHMOD_EXECUTE - 执行权限 (1)
*
* 注意: 该功能使用了 POSIX 扩展和 fileowner()、filegroup()函数. 如果其中任何被禁用, 此功能需要建立适当的权限, 通过调用 is_readable() 和 is_writable() 函数.
*
* @参数 字符串	$filename	文件或目录
* @参数 整数	$perms		权限设置
*
* @返回 布尔值	true 为成功, 否则 false
* @作者 faw, phpBB Group
*/
function phpbb_chmod($filename, $perms = CHMOD_READ)
{
	static $_chmod_info;

	// 如果不存在，直接返回 false
	if (!file_exists($filename))
	{
		return false;
	}

	// Determine some common vars
	if (empty($_chmod_info))
	{
		if (!function_exists('fileowner') || !function_exists('filegroup'))
		{
			// No need to further determine owner/group - it is unknown
			$_chmod_info['process'] = false;
		}
		else
		{

			// Determine owner/group of common.php file and the filename we want to change here
			$common_php_owner = @fileowner(ROOT_PATH . 'common.php');
			$common_php_group = @filegroup(ROOT_PATH . 'common.php');

			// And the owner and the groups PHP is running under.
			$php_uid = (function_exists('posix_getuid')) ? @posix_getuid() : false;
			$php_gids = (function_exists('posix_getgroups')) ? @posix_getgroups() : false;

			// If we are unable to get owner/group, then do not try to set them by guessing
			if (!$php_uid || empty($php_gids) || !$common_php_owner || !$common_php_group)
			{
				$_chmod_info['process'] = false;
			}
			else
			{
				$_chmod_info = array(
					'process'		=> true,
					'common_owner'	=> $common_php_owner,
					'common_group'	=> $common_php_group,
					'php_uid'		=> $php_uid,
					'php_gids'		=> $php_gids,
				);
			}
		}
	}

	if ($_chmod_info['process'])
	{
		$file_uid = @fileowner($filename);
		$file_gid = @filegroup($filename);

		// Change owner
		if (@chown($filename, $_chmod_info['common_owner']))
		{
			clearstatcache();
			$file_uid = @fileowner($filename);
		}

		// Change group
		if (@chgrp($filename, $_chmod_info['common_group']))
		{
			clearstatcache();
			$file_gid = @filegroup($filename);
		}

		// If the file_uid/gid now match the one from common.php we can process further, else we are not able to change something
		if ($file_uid != $_chmod_info['common_owner'] || $file_gid != $_chmod_info['common_group'])
		{
			$_chmod_info['process'] = false;
		}
	}

	// Still able to process?
	if ($_chmod_info['process'])
	{
		if ($file_uid == $_chmod_info['php_uid'])
		{
			$php = 'owner';
		}
		else if (in_array($file_gid, $_chmod_info['php_gids']))
		{
			$php = 'group';
		}
		else
		{
			// Since we are setting the everyone bit anyway, no need to do expensive operations
			$_chmod_info['process'] = false;
		}
	}

	// We are not able to determine or change something
	if (!$_chmod_info['process'])
	{
		$php = 'other';
	}

	// Owner always has read/write permission
	$owner = CHMOD_READ | CHMOD_WRITE;
	if (is_dir($filename))
	{
		$owner |= CHMOD_EXECUTE;

		// Only add execute bit to the permission if the dir needs to be readable
		if ($perms & CHMOD_READ)
		{
			$perms |= CHMOD_EXECUTE;
		}
	}

	switch ($php)
	{
		case 'owner':
			$result = @chmod($filename, ($owner << 6) + (0 << 3) + (0 << 0));

			clearstatcache();

			if (is_readable($filename) && phpbb_is_writable($filename))
			{
				break;
			}

		case 'group':
			$result = @chmod($filename, ($owner << 6) + ($perms << 3) + (0 << 0));

			clearstatcache();

			if ((!($perms & CHMOD_READ) || is_readable($filename)) && (!($perms & CHMOD_WRITE) || phpbb_is_writable($filename)))
			{
				break;
			}

		case 'other':
			$result = @chmod($filename, ($owner << 6) + ($perms << 3) + ($perms << 0));

			clearstatcache();

			if ((!($perms & CHMOD_READ) || is_readable($filename)) && (!($perms & CHMOD_WRITE) || phpbb_is_writable($filename)))
			{
				break;
			}

		default:
			return false;
		break;
	}

	return $result;
}

/**
* 删除文件或目录
* @参数 字符串 $dir_name 目录
* @返回 删除成功返回 true 否则 false
**/
function phpbb_deldir($dir_name)
{
	
	$dir = opendir($dir_name); 	
	
	while ( $file = readdir($dir) )
	{ 
		if( ($file != '.') && ($file != '..') )
		{ 
			$fullpath = $dir_name . '/' . $file; 
			if( !is_dir($fullpath) )
			{ 
				unlink($fullpath); 
			}
			else
			{ 
				phpbb_deldir($fullpath); 
			}	 
		} 
	} 
	
	closedir($dir);
	
	if( rmdir($dir_name) )
	{ 
		return true;
	}
	else
	{
		return false;
	}
}

function _set_var(&$result, $var, $type, $multibyte = false)
{
	settype($var, $type);
	$result = $var;

	if ($type == 'string')
	{
		$result = trim(htmlspecialchars(str_replace(array("\r\n", "\r", '\xFF'), array("\n", "\n", ' '), $result)));
		$result = stripslashes($result);
		if ($multibyte)
		{
			$result = preg_replace('#&amp;(\#[0-9]+;)#', '&\1', $result);
		}
	}
}

/**
* 取得 GET 或 POST 的值
* @参数 字符串 $var_name GET 或 POST 的变量名
* @参数 所有 $default 默认值
* @参数 布尔值 $multibyte 同时获得多个值，默认false，true为开启 
**/
function get_var($var_name, $default, $multibyte = false)
{
	global $_POST, $_GET;

	$request_var = (isset($_POST[$var_name])) ? $_POST : $_GET;

	if (!isset($request_var[$var_name]) || (is_array($request_var[$var_name]) && !is_array($default)) || (is_array($default) && !is_array($request_var[$var_name])))
	{
		return (is_array($default)) ? array() : $default;
	}

	$var = $request_var[$var_name];

	if (!is_array($default))
	{
		$type = gettype($default);
	}
	else
	{
		list($key_type, $type) = each($default);
		$type = gettype($type);
		$key_type = gettype($key_type);
	}

	if (is_array($var))
	{
		$_var = $var;
		$var = array();

		foreach ($_var as $k => $v)
		{
			if (is_array($v))
			{
				foreach ($v as $_k => $_v)
				{
					_set_var($k, $k, $key_type);
					_set_var($_k, $_k, $key_type);
					_set_var($var[$k][$_k], $_v, $type, $multibyte);
				}
			}
			else
			{
				_set_var($k, $k, $key_type);
				_set_var($var[$k], $v, $type, $multibyte);
			}
		}
	}
	else
	{
		_set_var($var, $var, $type, $multibyte);
	}
		
	return $var;
}

/*
* 伪静态的替换规则
*/
function phpbb_replace_mod_rewrite(&$ob_contents) 
{ 
	$urlin = array(
		"'viewtopic.php\?p=([1-9][0-9]{0,7})#([1-9][0-9]{0,7})(?!\&amp;)'", 
		"'viewtopic.php\?p=([1-9][0-9]{0,7})(?!\&amp;)'",
		"'viewtopic.php\?t=([1-9][0-9]{0,7})(?!\&amp;)'",
		"'viewforum.php\?f=([1-9][0-9]{0,7})(?!\&amp;)'",
		"'forum.php?c=([1-9][0-9]{0,7})(?!\&amp;)'",
	); 
	$urlout = array( 
		"viewpost-\\1-\\2.html",
		"viewposts-\\1.html",
		"topic-\\1.html",
		"forum-\\1.html",
		"forum-cat-\\1.html"
	); 
	
	$new_ob_contents = preg_replace($urlin, $urlout, $ob_contents); 
	
	return $new_ob_contents; 
}

/*
* @功能
* 虽然这样会增加数据库的负担，但是这样不会出现部分链接到用户中心时提示用户不存在
*/
function phpbb_message_at_link($message)
{
	preg_match_all("!(@|＠)([\\x{4e00}-\\x{9fa5}A-Za-z0-9_\\-]{1,12})(\x20|&nbsp;|<|\xC2\xA0|\r|\n|\x03|\t|,|\\?|\\!|:|;|，|。|？|！|：|；|、|…|$)!ue", $message, $matches);
	
	if (is_array($matches[2]))
	{
		foreach ($matches[2] as $username)
		{
			if ( !$row = get_userdata($username, true))
			{
				continue;
			}
			
			$message = str_replace('@' . $row['username'], '@<a href="' . append_sid('ucp.php?mode=viewprofile&amp;' . POST_USERS_URL . '=' . $row['user_id']) . '">' . $row['username'] . '</a>', $message);
			
		}
	}
	
	return $message;
}

function phpbb_message_at($message)
{
	global $db, $userdata, $user_ip;
	
	$at_url = basename($_SERVER['PHP_SELF']);
	
	if ($_POST[POST_TOPIC_URL] != '')
	{
		$at_url = 'viewtopic.php?t=' . abs(intval($_POST[POST_TOPIC_URL]));
	}
	else
	{
		$at_p = abs(intval($_POST[POST_POST_URL]));
		$at_url = 'viewtopic.php?p=' . $at_p . '#' . $at_p;
	}

	preg_match_all("!(@|＠)([\\x{4e00}-\\x{9fa5}A-Za-z0-9_\\-]{1,12})(\x20|&nbsp;|<|\xC2\xA0|\r|\n|\x03|\t|,|\\?|\\!|:|;|，|。|？|！|：|；|、|…|$)!ue", $message, $matches);

	$atuser = array_count_values($matches[2]);

	$atuser = array_keys($atuser);//删除重复值

	for ($i = 0; $i < count($atuser); $i++)
	{

		if ( !$row = get_userdata($atuser[$i], true))
		{
			continue;//用户不存在，跳过这次循环
		}

		$to_userdata = $row['user_id'];

		$msg_time = time();

		$sql_info = 'INSERT INTO ' . PRIVMSGS_TABLE . " (privmsgs_type, privmsgs_subject, privmsgs_from_userid, privmsgs_to_userid, privmsgs_date, privmsgs_ip) 
			VALUES (" . PRIVMSGS_NEW_MAIL . ", '系统@消息', " . $userdata['user_id'] . ", " . $row['user_id'] . ", $msg_time, '$user_ip')";
		
		if ( !$db->sql_query($sql_info) )
		{
			message_die(GENERAL_ERROR, '无法插入数据到信息表', '', __LINE__, __FILE__, $sql);
		}

		$sql = 'SELECT privmsgs_id 
			FROM ' . PRIVMSGS_TABLE . ' 
			WHERE privmsgs_id = (SELECT MAX(privmsgs_id) FROM ' . PRIVMSGS_TABLE . ')';

		if (!$result = $db->sql_query($sql))
		{
			message_die(GENERAL_ERROR, '无法获取最后privmsgs_id', '', __LINE__, __FILE__, $sql);
		}

		$msgrow = $db->sql_fetchrow($result);

		$sql = 'UPDATE ' . USERS_TABLE . ' 
			SET user_new_privmsg = user_new_privmsg + 1 
			WHERE user_id = ' . $row['user_id'];

		if ( !$db->sql_query($sql) )
		{
			message_die(GENERAL_ERROR, '无法更新 ' . USERS_TABLE . ' 表', '', __LINE__, __FILE__, $sql);
		}

		$privmsg_sent_id = $msgrow['privmsgs_id'];

		$at_message = '您收到一条来自系统的@消息，您可以点击 <a href="' . $at_url . '">这里</a> 查看';

		$sql = 'INSERT INTO ' . PRIVMSGS_TEXT_TABLE . " (privmsgs_text_id, privmsgs_text)
		   VALUES ($privmsg_sent_id, '" . str_replace("\'", "''", $at_message) . "')";

		if ( !$db->sql_query($sql) )
		{
			message_die(GENERAL_ERROR, '无法插入数据到 privmsgs_text ', '', __LINE__, __FILE__, $sql);
		}

	}

	return $message;
}

/*
* 对风格安装包的cfg文件进行解析
*/
function parse_cfg_file($filename, $lines = false)
{
	$parsed_items = array();

	if ($lines === false)
	{
		$lines = file($filename);
	}

	foreach ($lines as $line)
	{
		$line = trim($line);

		if (!$line || $line[0] == '#' || ($delim_pos = strpos($line, '=')) === false)
		{
			continue;
		}

		// Determine first occurrence, since in values the equal sign is allowed
		$key = strtolower(trim(substr($line, 0, $delim_pos)));
		$value = trim(substr($line, $delim_pos + 1));

		if (in_array($value, array('off', 'false', '0')))
		{
			$value = false;
		}
		else if (in_array($value, array('on', 'true', '1')))
		{
			$value = true;
		}
		else if (!trim($value))
		{
			$value = '';
		}
		else if (($value[0] == "'" && $value[sizeof($value) - 1] == "'") || ($value[0] == '"' && $value[sizeof($value) - 1] == '"'))
		{
			$value = substr($value, 1, sizeof($value)-2);
		}

		$parsed_items[$key] = $value;
	}
	
	if (isset($parsed_items['inherit_from']) && isset($parsed_items['name']) && $parsed_items['inherit_from'] == $parsed_items['name'])
	{
		unset($parsed_items['inherit_from']);
	}

	return $parsed_items;
}

function obtain_images_data()
{
	global $template;

	$filename = phpbb_realpath($template->root . '/imageset.cfg');

	if (!file_exists($filename))
	{
		$filename = $template->root . 'styles/' . $template->fixed_style . $filename;
		if( !file_exists($filename) )
		{
			trigger_error("Template->make_filename(): Error - file $filename does not exist", E_USER_ERROR);
		}
	}

	$image_array = array();
	$newdata = parse_cfg_file($filename);

	foreach ($newdata as $image_name => $value)
	{
		if (strpos($value, '*') !== false)
		{
			if (substr($value, -1, 1) === '*')
			{
				list($image_filename, $image_height) = explode('*', $value);
				$image_width = 0;
			}
			else
			{
				list($image_filename, $image_height, $image_width) = explode('*', $value);
			}
		}
		else
		{
			$image_filename = $value;
			$image_height = $image_width = 0;
		}

		if (strpos($image_name, 'img_') === 0 && $image_filename)
		{
			$image_name = substr($image_name, 4);
			$image_array[$image_name] = array(
				'image_filename'	=> (string) $image_filename,
				'image_height'		=> (int) $image_height,
				'image_width'		=> (int) $image_width
			);
		}
	}				

	return $image_array;
}

/*
* 生成网站的图标
*/
function make_style_image($image_name, $image_title = '', $image_alt = '.', $other_property = '')
{
	global $style_image_array, $global_style_path;

	$image = '<img src="" alt="."/>';
	
	if (empty($style_image_array))
	{
		$image = '<img src="" alt="."/>';
	}

	if(isset($style_image_array[$image_name]))
	{
		$image = '<img src="' . ROOT_PATH . 'styles/' . 
		$global_style_path . '/images/' . $style_image_array[$image_name]['image_filename'] 
		. '" width="' . $style_image_array[$image_name]['image_width'] 
		. '" height="' . $style_image_array[$image_name]['image_height'] 
		. '" title="' . $image_title 
		. '" alt="' . $image_alt
		. '" ' . $other_property . '/>';
	}

	return $image;
}

?>